require_relative '../../spec/spec_helper'

module MdrTest
  class InteractiveApi
    include RSpec::Expectations
    include RSpec::Matchers
    def initialize
      super
      @error = String.new('')
      @failures = []
      @data = []
    end

    def executeAndFetchFlexReportQuery(service, headers, interval, from_date, to_date, query, limit, page_size, time_bounded, report, check_for_error = 'false', relative_time_range_days=nil)
      if relative_time_range_days.nil?
        time_range = { "from": from_date, "to": to_date }
      else
        max_values_relative_time_range = {'DAILY': 90, 'HOURLY': 7, 'MONTHLY': 12, 'WEEKLY': 52}.to_h
        if relative_time_range_days > max_values_relative_time_range[interval.to_sym]
          puts "Error : Invalid Value passed for Relative Time Range Days for interval #{interval} Expected: < #{max_values_relative_time_range[interval.to_sym]} Actual: #{relative_time_range_days}"
          @failures << "Error : Invalid Value passed for Relative Time Range Days for interval #{interval} Expected: < #{max_values_relative_time_range[interval.to_sym]} Actual: #{relative_time_range_days}"
          return @failures
        end
        time_range =  { "last": relative_time_range_days }
      end
      @variables = if time_bounded
        {
          "pageSize": page_size,
          "queryInput": {
            "sqlStatement": query,
            "dataGranularity": interval,
            "needBackLinkingForTags": false,
            "timeRange": time_range,
            "limit": limit
          }
        }
      elsif !time_bounded && report
        {
          "pageSize": page_size,
          "queryInput": {
            "sqlStatement": query,
            "limit": limit
          }
        }
      elsif time_bounded && !report
        {
          "pageSize": page_size,
          "queryInput": {
            "sqlStatement": query,
            "needBackLinkingForTags": false,
            "timeRange": time_range,
            "limit": limit
          }
        }
      else
        {
          "pageSize": page_size,
          "queryInput": {
            "sqlStatement": query,
            "limit": limit
          }
        } end
      wrap_request do
        graphql url(service: service),
                format_request(
                  :executeFlexReportQuery,
                  service: service
                ),
                @variables, headers
        if check_for_error.eql? 'true'
          begin
            return json_body[:errors][0][:message]
          rescue NoMethodError
            return "Test Case Was Expecting an Exception on the API for the negative case with Proper error Message But No Errors appeared in API Call"
          end
        end

        expect_success_graphql unless check_for_error.eql? 'true'

        # check(:executeFlexReportQuery, service: service)
        expect(json_body[:data][:executeFlexReportQuery][:page]).not_to eq(nil)
        page = json_body[:data][:executeFlexReportQuery][:page]
        variables = {
          'input': {
            'page': page
          }
        }
        puts "Input for getFlexReportQueryResult : #{variables}"
        @retry_count = 0
        begin
          puts "Fetch FlexReportQueryResult with retry count: #{@retry_count}"
          graphql url(service: service), format_request(:getFlexReportQueryResult, service: service),
                  variables, headers
          expect_success_graphql
        rescue NoMethodError => error
          @retry_count += 1
          retry if @retry_count < 3
          raise("Error Occurred While Fetching getFlexReportQueryResult with error #{error.message}") if @retry_count >= 3
        end
        # check(:getFlexReportQueryResult, service: service)
        raise("#{json_body[:errors][0][:message]} while Fetching Next Page Response") if json_body.keys.include? 'errors' && json_body[:data][:getFlexReportQueryResult].nil?
        @failures << "Unable To Parse Column Metadata in the First page response" if json_body[:data][:getFlexReportQueryResult][:columnMetadata].empty?
        @failures << "Rows Result Returned empty array in First page response , when expected to have data" if json_body[:data][:getFlexReportQueryResult][:rows].empty?
        @next_page = json_body[:data][:getFlexReportQueryResult][:pageInfo][:nextPage] unless json_body[:data][:getFlexReportQueryResult][:pageInfo][:nextPage].nil?
        @data << json_body[:data][:getFlexReportQueryResult][:rows]
        unless json_body[:data][:getFlexReportQueryResult][:pageInfo][:nextPage].nil?
          loop do

            page = @next_page

            variables = {
              'input': {
                'page': page
              }
            }
            @retry_count = 0
            begin
              puts "Fetch Subsequent Pages After Fetching First page Response with retry count: #{@retry_count}"
              graphql url(service: service), format_request(:getFlexReportQueryResult, service: service),
                      variables, headers
              expect_success_graphql
            rescue NoMethodError => error
              @retry_count += 1
              retry if @retry_count < 3
              raise("Error Occurred While Fetching getFlexReportQueryResult with error #{error.message}") if @retry_count >= 3
            end
            # check(:getFlexReportQueryResult, service: service)
            raise("#{json_body[:errors][0][:message]} while Fetching Next Page Response") if json_body.keys.include? 'errors' && json_body[:data][:getFlexReportQueryResult].nil?
            @failures << "Unable To Parse Column Metadata in the next page response" if json_body[:data][:getFlexReportQueryResult][:columnMetadata].empty?
            @next_page = json_body[:data][:getFlexReportQueryResult][:pageInfo][:nextPage]
            if json_body[:data][:getFlexReportQueryResult][:rows].empty?
              @failures << "Rows Result Returned empty array in next page, when expected to have data" unless @next_page.nil?
            end
            @data << json_body[:data][:getFlexReportQueryResult][:rows]
            break if @next_page.nil?
          end
        end
      end
      @failures << "Expected Data to be returned in the response" if @data.nil?
      @failures if check_for_error.eql?('false')
    end
  end
end
