require 'spec_helper'
require 'securerandom'
include ChtApitestMdr::MdrHelpers

module MdrFlexReportTest

  describe 'FlexReports', :async_apis do
    include_context 'request metadata'

    before(:all) do
      @report_name = "ALIAS TABLE VALIDATION - #{ENV['NAMESPACE']} - #{rand(999999999999999)}"
      @multi_part_api = MultiPartQueryApi.new
      @async_preview_ids = []
      @report_ids = []
      @dataset = 'AWS_CUR'
      start_time_day = Time.new - 11540000
      @from_month = (start_time_day).strftime("%Y-%m")
      @to_month = (start_time_day+13328000).strftime("%Y-%m")
      @from_month_daily_date = start_time_day.strftime("%Y-%m-%d")
      @to_month_daily_date = (start_time_day+7689600).strftime("%Y-%m-%d")
      @aggregated_sql = "with cxtemp_aws_cur as ( select aws1.\"LineItem_UnblendedCost\" as Cost, aws1.TimeInterval_Day as Day, aws1.\"perspective##Team\" AS Perspective_Team, 'aws' as CloudType FROM AWS_CUR as aws1 ), cxtemp_azure as ( select azurecu.\"CostInBillingCurrency\" as Cost, azurecu.TimeInterval_day as Day, azurecu.\"perspective##Team\" AS Perspective_Team, 'azure' as CloudType FROM AZURE_COST_USAGE  as azurecu ), cxtemp_gcp as ( SELECT gcp1.timeInterval_Day AS Day, gcp1.\"Cost\" AS Cost, 'gcp' as CloudType FROM GCP_BILLING_EXPORT as gcp1 WHERE (gcp1.\"Project_Id\" IS NOT NULL) AND (gcp1.\"Cost_Type\" LIKE '%regular%') ), cxtemp_aws_azure as ( select SUM(cxaws.Cost) as AwsCost, SUM(cxaws.Cost) as AzureCost, cxaws.Day as Day, cxaws.Perspective_Team as P_Team FROM cxtemp_aws_cur as cxaws LEFT JOIN cxtemp_azure ON cxtemp_azure.Day = cxaws.Day WHERE cxaws.Perspective_Team = cxtemp_azure.Perspective_Team GROUP BY cxaws.Day, cxtemp_azure.Day, cxaws.Perspective_Team ) select tempawsazure.Day as Day, tempawsazure.P_Team as Perspective_Team, ROUND(SUM(cxtemp_gcp.Cost), 2) as GcpCost, ROUND(SUM(tempawsazure.Awscost), 2) as AwsCost, ROUND(SUM(tempawsazure.AzureCost), 2) as AzureCost FROM cxtemp_aws_azure as tempawsazure LEFT JOIN cxtemp_gcp ON cxtemp_gcp.Day = tempawsazure.Day GROUP BY tempawsazure.Day, tempawsazure.P_Team"
      @default_x = "Day"
      @default_ys = ["GcpCost"]
      @default_z = "Perspective_Team"
      @z = %w[GcpCost]
      @y = %w[GcpCost]
      @z = ["Perspective_Team"]
      @default_chart_type = 'column-stacked'
    end

    it 'Create Report With Chart Config Selected(Column-Stacked) with DAILY Granularity', :create do
      wrap_request do
        variables = {
          'input': {
            'name': @report_name,
            'notification': {
              'sendUserEmail': false,
              'otherEmailIds': []
            },
            'chart': {
              'columns': [],
              'y': @y,
              'defaultX': @default_x,
              'defaultYs': @default_ys,
              'defaultZ': @default_z,
              'defaultChartType': @default_chart_type
            },
            'query': {
              'dataGranularity': 'DAILY',
              'limit': -1,
              'needBackLinkingForTags': false,
              'sqlStatement': @aggregated_sql,
              'timeRange': {
                'from': @from_month_daily_date,
                'to': @to_month_daily_date
              }
            }
          }
        }
        graphql url(service: service),
                format_request(
                  :createFlexReport,
                  service: service
                ),
                variables, headers
        expect_success_graphql
        @report_ids << json_body[:data][:createFlexReport][:id]
        expect(%w[QUEUED RUNNING]).to include(json_body[:data][:createFlexReport][:result][:status])
        expect(json_body[:data][:createFlexReport][:query][:dataGranularity]).to eql 'DAILY'
      end
    end

    it 'Can Poll the Status of the Report Created', :status do
      poll_start_time = Time.now.to_i
      loop do
        wrap_request do
          variables = {}
          graphql url(service: service),
                  format_request(
                    :node,
                    service: service,
                    crn: @report_ids.first,
                    type: 'FlexReport',
                    fields: 'name archived result { contentType contents { preSignedUrl } message reportUpdatedOn status } chart { columns { columnUnit dataType filterable name namespace type usageAffinity } x y z defaultX defaultYs defaultZ defaultChartType } query { sqlStatement dataGranularity timeRange { from to } }'
                  ),
                  variables, headers
          expect_success_graphql
          check(:node, service: service)
        end
        puts "Polling the Report with Report Name: #{json_body[:data][:node][:name]}, Status: #{json_body[:data][:node][:result][:status]}"
        # In Staging there won't be much load so checking if the report can be completed under 5 mins, Increase this to 10 mins if the PR Builds
        # FAil Constantly at this test case with Runtime Execption with below message
        raise("Error Occurred While Polling the Status of Report: Report #{json_body[:data][:node][:name]} is in #{json_body[:data][:node][:result][:status]} state for more than 5 mins") if Time.now.to_i > poll_start_time + 300000
        break unless %w[QUEUED RUNNING].include? json_body[:data][:node][:result][:status]
      end
      expect(json_body[:data][:node][:id]).to eql @report_ids.first
      expect(json_body[:data][:node][:name]).to eq  @report_name
      expect(json_body[:data][:node][:result][:status]).to eql 'COMPLETED'
      expect(json_body[:data][:node][:result][:status]).not_to eql 'FAILED'
      expect(json_body[:data][:node][:chart][:defaultX]).to eql @default_x
      expect(json_body[:data][:node][:chart][:defaultChartType]).to eql @default_chart_type
      expect(json_body[:data][:node][:chart][:defaultYs]).to eql @default_ys
      expect(json_body[:data][:node][:chart][:defaultZ]).to eql @default_z
    end

    it 'can query on top of the Report created getting the async preview id ', chunked: true do
      @temp_table_sql = "with cxtemp_aws_cur as ( select aws1.\"LineItem_UnblendedCost\" as Cost, aws1.TimeInterval_Day as Day, aws1.\"perspective##Team\" AS Perspective_Team, 'aws' as CloudType FROM AWS_CUR as aws1 ), cxtemp_azure as ( select azurecu.\"CostInBillingCurrency\" as Cost, azurecu.TimeInterval_day as Day, azurecu.\"perspective##Team\" AS Perspective_Team, 'azure' as CloudType FROM AZURE_COST_USAGE  as azurecu ), cxtemp_gcp as ( SELECT gcp1.timeInterval_Day AS Day, gcp1.\"Cost\" AS Cost, 'gcp' as CloudType FROM GCP_BILLING_EXPORT as gcp1 WHERE (gcp1.\"Project_Id\" IS NOT NULL) AND (gcp1.\"Cost_Type\" LIKE '%regular%') ), cxtemp_aws_azure as ( select SUM(cxaws.Cost) as AwsCost, SUM(cxaws.Cost) as AzureCost, cxaws.Day as Day, cxaws.Perspective_Team as P_Team FROM cxtemp_aws_cur as cxaws LEFT JOIN cxtemp_azure ON cxtemp_azure.Day = cxaws.Day WHERE cxaws.Perspective_Team = cxtemp_azure.Perspective_Team GROUP BY cxaws.Day, cxtemp_azure.Day, cxaws.Perspective_Team ) select tempawsazure.Day as Day, tempawsazure.P_Team as Perspective_Team, ROUND(SUM(cxtemp_gcp.Cost), 2) as GcpCost, ROUND(SUM(tempawsazure.Awscost), 2) as AwsCost, ROUND(SUM(tempawsazure.AzureCost), 2) as AzureCost FROM cxtemp_aws_azure as tempawsazure LEFT JOIN cxtemp_gcp ON cxtemp_gcp.Day = tempawsazure.Day GROUP BY tempawsazure.Day, tempawsazure.P_Team"
      wrap_request do
        variables = {
          'chunkedResult': true,
          'queryInput': {
            'dataGranularity': 'DAILY',
            'sqlStatement': @temp_table_sql,
            'limit': 5000,
            'timeRange': {
              'from': @from_month_daily_date,
              'to': @to_month_daily_date
            }
          }
        }
        graphql url(service: service),
                format_request(
                  :asyncPreviewFlexReport,
                  service: service
                ),
                variables, headers
        expect_success_graphql
        check(:asyncPreviewFlexReport, service: service)
        expect(json_body[:data][:asyncPreviewFlexReport][:mode]).to eq 'OVERRIDE'
        expect(json_body[:data][:asyncPreviewFlexReport][:status]).to eq 'RUNNING'
        expect(json_body[:data][:asyncPreviewFlexReport][:asyncPreviewId]).not_to eq ''
        expect(json_body[:data][:asyncPreviewFlexReport][:message]).to eq ''
        expect(json_body[:data][:asyncPreviewFlexReport][:percentCompleted]).to eq 0
        expect(json_body[:data][:asyncPreviewFlexReport][:preview][:csv]).to eq ''
        @async_preview_ids << json_body[:data][:asyncPreviewFlexReport][:asyncPreviewId]
      end
    end

    it 'checks the aysncPreviewFlexReport api request response is COMPLETED using asyncPreviewFlexReportResponse api' do
      wrap_request do
        variables = {
          'statusInput': {
            'asyncPreviewId': @async_preview_ids.first
          }
        }
        graphql url(service: service),
                format_request(
                  :asyncPreviewFlexReportResponseStatus,
                  service: service
                ),
                variables, headers
        expect_success_graphql
        check(:asyncPreviewFlexReportResponseStatus, service: service)
        @async_preview_ids << json_body[:data][:asyncPreviewFlexReportResponseStatus][:asyncPreviewId]
      end
      async_preview_id = @async_preview_ids.last
      loop do
        break if async_preview_id.nil?
        @json_body = @multi_part_api.poll_async_preview_api_response_status(async_preview_id, service, headers)
        break unless ['RUNNING', 'CHUNKED'].include? @json_body[:data][:asyncPreviewFlexReportResponseStatus][:status]
        async_preview_id = @json_body[:data][:asyncPreviewFlexReportResponseStatus][:asyncPreviewId]
      end
      expect(@json_body[:data][:asyncPreviewFlexReportResponseStatus][:status]).to eq('COMPLETED')
      expect(@json_body[:data][:asyncPreviewFlexReportResponseStatus][:mode]).to eq 'OVERRIDE'
      expect(@json_body[:data][:asyncPreviewFlexReportResponseStatus][:asyncPreviewId]).to eq nil
      expect(@json_body[:data][:asyncPreviewFlexReportResponseStatus][:message]).to eq ''
      expect(@json_body[:data][:asyncPreviewFlexReportResponseStatus][:percentCompleted]).to eq 100
      expect(@json_body[:data][:asyncPreviewFlexReportResponseStatus][:preview][:csv]).not_to eq ''
    end

    it 'can clean up the above created report using deleteFlexReport' do
      report_id = @report_ids.first
      wrap_request do
        variables = {
          'id': report_id
        }
        graphql url(service: service),
                format_request(
                  :delete,
                  service: service
                ),
                variables, headers
        expect_success_graphql
        check(:delete, service: service)
        expect(json_body[:data][:delete]).to eql true
      end
    end


  end
end
