require 'spec_helper'

module MdrTest
  describe 'Mdr executeFlexReportQuery' do
    include_context 'request metadata'

    before(:all) do
      start_time_day = Time.new - 31540000
      @from_month = (start_time_day).strftime("%Y-%m")
      @to_month = (start_time_day+23328000).strftime("%Y-%m")
      @from_month_daily_date = start_time_day.strftime("%Y-%m-%d")
      @to_month_daily_date = (start_time_day+7689600).strftime("%Y-%m-%d")
      @failures = []
      @data = []
      @interactive_api = InteractiveApi.new
    end

    before(:each) do
      @headers = headers
    end

    it "Execute Paginated Azure can executeFlexReportQuery and getFlexReportQueryResult for", :swami do
      # TODO: ENABLE AZURE PRICING TEST CASES ONCE https://cloudhealthtech.atlassian.net/browse/KAZ-2156 resolved , then remove the below skip
      skip("Skipping The Test Execution in Preprod as Azure Metadata Pricing is not yet ingested") if ENV['TEST_ENV'].eql? 'preprod'
      query = "select armSkuName as name, armRegionName as region,  retailPrice as retailPrice, serviceName as serviceName,skuName as skuname from DX_PRICING_AZURE"
      failures = @interactive_api.executeAndFetchFlexReportQuery(service, @headers, 'MONTHLY', @from_month, @to_month, query, 5000, 100, false, false, 'false' )
      expect(failures).to be_empty
    end

    it "Execute Paginated Azure-Metadata can executeFlexReportQuery and getFlexReportQueryResult for", :swami do
      # TODO: ENABLE AZURE PRICING TEST CASES ONCE https://cloudhealthtech.atlassian.net/browse/KAZ-2156 resolved , then remove the below skip
      skip("Skipping The Test Execution in Preprod as Azure Metadata Pricing is not yet ingested") if ENV['TEST_ENV'].eql? 'preprod'
      query = "select name as VM, capabilities_#_vcpus as VCPUs, capabilities_#_memorygb as MemoryInGB, capabilities_#_maxresourcevolumemb as Storage, Capabilities as metadata from DX_PRICING_METADATA_AZURE where resourceType ='virtualMachines'"
      failures = @interactive_api.executeAndFetchFlexReportQuery(service, @headers, 'MONTHLY', @from_month, @to_month, query, 5000, 100, false, false, 'false' )
      expect(failures).to be_empty
    end

    it "Execute Paginated  and Azure-Metadata and pricing join can executeFlexReportQuery and getFlexReportQueryResult for", :swami do
      # TODO: ENABLE AZURE PRICING TEST CASES ONCE https://cloudhealthtech.atlassian.net/browse/KAZ-2156 resolved , then remove the below skip
      skip("Skipping The Test Execution in Preprod as Azure Metadata Pricing is not yet ingested") if ENV['TEST_ENV'].eql? 'preprod'
      query = "with cxtemp_azure as (select armskuname as armskuname, armRegionName as armRegionName , retailPrice as retailPrice from dx_pricing_azure where servicename = 'Virtual Machines') select  cxtemp_azure.armskuname as armskuname, cxtemp_azure.armRegionName as armRegionName, cxtemp_azure.retailPrice as retailPrice , capabilities_#_memorygb as memory, capabilities_#_vcpus as vcpu from dx_pricing_metadata_azure  join cxtemp_azure  on cxtemp_azure.armskuname=name where  resourcetype='virtualMachines'"
      failures = @interactive_api.executeAndFetchFlexReportQuery(service, @headers, 'MONTHLY', @from_month, @to_month, query, 5000, 100, false, false, 'false' )
      expect(failures).to be_empty
    end

    it "Execute Preview Azure Pricing  can executeFlexReportQuery and getFlexReportQueryResult for", :swami do
      # TODO: ENABLE AZURE PRICING TEST CASES ONCE https://cloudhealthtech.atlassian.net/browse/KAZ-2156 resolved , then remove the below skip
      skip("Skipping The Test Execution in Preprod as Azure Metadata Pricing is not yet ingested") if ENV['TEST_ENV'].eql? 'preprod'
      query = "select name name, family family , StartDate startdate, EndDate endDate from dx_pricing_metadata_azure"
      failures = @interactive_api.executeAndFetchFlexReportQuery(service, @headers, 'MONTHLY', @from_month, @to_month, query, 5000, 100, false, false, 'false' )
      expect(failures).to be_empty
      expect_success_graphql
    end

    it "Athena Execute Paginated  and Azure-Metadata and pricing join can executeFlexReportQuery and getFlexReportQueryResult for", :swami do
      # TODO: ENABLE AZURE PRICING TEST CASES ONCE https://cloudhealthtech.atlassian.net/browse/KAZ-2156 resolved , then remove the below skip
      skip("Skipping The Test Execution in Preprod as Azure Metadata Pricing is not yet ingested") if ENV['TEST_ENV'].eql? 'preprod'
      query = "--athena with cxtemp_azure as (select armskuname as armskuname, armRegionName as armRegionName , retailPrice as retailPrice from dx_pricing_azure where servicename = 'Virtual Machines') select  cxtemp_azure.armskuname as armskuname, cxtemp_azure.armRegionName as armRegionName, cxtemp_azure.retailPrice as retailPrice , capabilities_#_memorygb as memory, capabilities_#_vcpus as vcpu from dx_pricing_metadata_azure  join cxtemp_azure  on cxtemp_azure.armskuname=name where  resourcetype='virtualMachines'"
      failures = @interactive_api.executeAndFetchFlexReportQuery(service, @headers, 'MONTHLY', @from_month, @to_month, query, 5000, 100, false, false, 'false' )
      expect(failures).to be_empty
      expect_success_graphql
    end

   #TODO: uncomment the test case when starburst staging is up and working
    xit "Presto Execute preview Azure Pricing  can executeFlexReportQuery and getFlexReportQueryResult for", :swami do
      # TODO: ENABLE AZURE PRICING TEST CASES ONCE https://cloudhealthtech.atlassian.net/browse/KAZ-2156 resolved , then remove the below skip
      skip("Skipping The Test Execution in Preprod as Azure Metadata Pricing is not yet ingested") if ENV['TEST_ENV'].eql? 'preprod'
      query = "--starburst select name name, family family , StartDate startdate, EndDate endDate from dx_pricing_metadata_azure"
      failures = @interactive_api.executeAndFetchFlexReportQuery(service, @headers, 'MONTHLY', @from_month, @to_month, query, 5000, 100, false, false, 'false' )
      expect(failures).to be_empty
      expect_success_graphql
    end

    it "Trino Execute review Azure Pricing can executeFlexReportQuery and getFlexReportQueryResult for", :swami do
      # TODO: ENABLE AZURE PRICING TEST CASES ONCE https://cloudhealthtech.atlassian.net/browse/KAZ-2156 resolved , then remove the below skip
      skip("Skipping The Test Execution in Preprod as Azure Metadata Pricing is not yet ingested") if ENV['TEST_ENV'].eql? 'preprod'
      query = "--trino select name name, family family , StartDate startdate, EndDate endDate from dx_pricing_metadata_azure"
      failures = @interactive_api.executeAndFetchFlexReportQuery(service, @headers, 'MONTHLY', @from_month, @to_month, query, 5000, 100, false, false, 'false' )
      expect(failures).to be_empty
      expect_success_graphql
    end
  end
end
