require 'spec_helper'
include ChtApitestMdr::MdrHelpers

module MdrFlexReportTest
  describe 'FlexReports' do
    include_context 'request metadata'
    before(:all) do
      @dataset = 'AWS_COST_HISTORY'
    end

    it "can FETCH METADATA for dataset AWS_COST_HISTORY" do
      @headers = headers
      @headers = pre_prod_headers if ENV['TEST_ENV'].eql? 'preprod'
      failures = []
      wrap_request do
        variables = {
          dataset: @dataset
        }
        graphql url(service: service), format_request(:dataSourceMetadata, service: service), variables, @headers
        expect_success_graphql
        check(:dataSourceMetadata, service: service)
        project_cost_columns_by_granualrity = Hash.new
        columns_by_granularity = []
        common_columns = []
        common_reference_datasets_datasource_info = []
        common_reference_datasets_datasource_name = []
        json_body[:data][:dataSourceMetadata][:columnsByGranularity].each do |column_details|
          column_details[:columns].each do |column|
            if column[:namespace].eql? 'timeInterval'
              columns_by_granularity << column[:name]
            end
          end
          columns_by_granularity = columns_by_granularity.uniq
          column_details[:columns].each do |column|
            if column[:namespace].eql? 'Cost'
              unless !column[:name].include? 'projected'
                project_cost_columns_by_granualrity[column_details[:dataGranularity]] = column[:name]
              end
            end
          end
        end
        json_body[:data][:dataSourceMetadata][:commonColumns].each do |column_info|
          common_columns << column_info[:name]
        end
        json_body[:data][:dataSourceMetadata][:commonReferenceDatasets].each do |dataset_info|
          common_reference_datasets_datasource_info << dataset_info[:dataSource]
          common_reference_datasets_datasource_name << dataset_info[:name]
        end
        supported_granualrities = json_body[:data][:dataSourceMetadata][:supportedGranularities]

        failures << "Common Reference datasets datasource names are not matching" unless %w[Services Charge-Type Region BillingAccount Account].sort.eql? common_reference_datasets_datasource_name.sort
        failures << "Common Reference datasets datasource names are not matching" unless %w[DX_AWS_SERVICE_CATEGORY DX_AWS_CHARGE_TYPE DX_AWS_REGION DX_AWS_BILLING_ACCOUNT DX_AWS_ACCOUNT].sort.eql? common_reference_datasets_datasource_info.sort
        failures << "Expected 4 Granularities , Missing some granularities" unless supported_granualrities.sort.eql? %w[DAILY HOURLY MONTHLY WEEKLY].sort
        failures << "Expected timeInterval Columns for all 4 granualrities, timeinterval columns are: #{columns_by_granularity}" unless columns_by_granularity.sort.eql? %w[timeInterval_Hour timeInterval_Month timeInterval_Day timeInterval_Week].sort
        failures << "Dataset metadata fetched for different dataset than input dataset" unless json_body[:data][:dataSourceMetadata][:dataset].eql? @dataset
        # failures << "Projected Cost Columns are Appearing For Unexpected Granularities" unless project_cost_columns_by_granualrity.keys.sort!.eql? ['DAILY', 'MONTHLY'].sort!
        # failures << "Projected Cost Columns name is incorrect" unless project_cost_columns_by_granualrity.values.uniq.first.eql? 'cost_projected'
        expect(failures).to eq([])
      end
    end
  end
end