require 'spec_helper'

module MdrTest
  describe 'Mdr dataSources' do
    include_context 'request metadata'

    before(:all) do
      @datasources = []
      start_time_day = Time.new - 31540000
      start_time_day = Time.new - 3154000 if ENV['TEST_ENV'].eql? 'preprod'
      @from_month = (start_time_day).strftime("%Y-%m")
      @to_month = (start_time_day+23328000).strftime("%Y-%m")
      @from_month_daily_date = start_time_day.strftime("%Y-%m-%d")
      @to_month_daily_date = (start_time_day+7689600).strftime("%Y-%m-%d")
      @failures = []
      @data = []
      @interactive_api = InteractiveApi.new
    end

    it 'can fetch all datasources available for views' do
      wrap_request do
        variables = {}
        graphql url(service: service),
                format_request(
                  :dataSources,
                  service: service
                ),
                variables, headers
        expect_success_graphql
        check(:dataSources, service: service)
        @views_from_response = json_body[:data][:dataSources].select { |datasource| datasource[:type].eql? 'VIEW' }.map { |datasource| datasource[:datasetName] }
      end
      views = %w[Azure_FullyLoaded_Cost AWS_Cost_Anomaly_RCA ]
      views = %w[AWS_Cost_Anomaly_RCA Aws_Kubernetes_Cost_History] if ENV['TEST_ENV'].eql? 'preprod'
      views.each do |view_name|
        wrap_request do
          variables = { dataset: view_name, type: 'VIEW' }
          graphql url(service: service),
                  format_request(
                    :dataSourceMetadata,
                    service: service
                  ),
                  variables, headers
          expect_success_graphql
          check(:dataSourceMetadata, service: service)
          @view_name = json_body[:data][:dataSourceMetadata][:dataset]
          @X = json_body[:data][:dataSourceMetadata][:chart][:defaultX]
          @Y = json_body[:data][:dataSourceMetadata][:chart][:defaultYs]
          @Z = json_body[:data][:dataSourceMetadata][:chart][:defaultZ]
          value_Y = @Y.first()
          query = "SELECT #{@X} AS #{@X}, SUM(#{value_Y}) AS SUM_#{value_Y} FROM #{view_name} GROUP BY #{@X}" if @Z.eql? ""
          query = "SELECT #{@X} AS #{@X}, SUM(#{value_Y}) AS SUM_#{value_Y} FROM #{view_name} GROUP BY #{@X},#{@Z}" unless @Z.eql? ""
          failures = @interactive_api.executeAndFetchFlexReportQuery(service, headers, 'MONTHLY', @from_month, @to_month, query, 5000, 100, true, false, 'false' )
          expect(failures).to be_empty
        end
      end
      expect(@failures).to eq([])
    end
  end
end
