require 'spec_helper'

module MdrTest
  describe 'Mdr executeFlexReportBulkQuery and getFlexReportBulkQueryResult' do
    include_context 'request metadata'

    # basic test to verify workflow used by reservation-recommendations service works
    it 'execute and get bulk query result (reservation recommendations scenario), verify job completion' do
      wrap_request do
        variables = {
          queryInput: {
            limit: -1,
            sqlStatement: "Select RowId as RowId from DOT",
          },
        }
        graphql url(service: service),
          format_request(
            :executeFlexReportBulkQuery,
            service: service
          ),
          variables, headers
        expect_success_graphql
        check(:executeFlexReportBulkQuery, service: service)
      end
      query_id = json_body[:data][:executeFlexReportBulkQuery][:queryExecutionId]

      wrap_request do
        variables = {
          flexReportExecutionId: query_id
        }
        graphql url(service: service),
          format_request(
            :getFlexReportBulkQueryResult,
            service: service
          ),
          variables, headers
        expect_success_graphql
        check(:getFlexReportBulkQueryResult, service: service)
        expect(json_body[:data][:getFlexReportBulkQueryResult][:queryStatus]).to_not eq("FAILED")
      end
      status = json_body[:data][:getFlexReportBulkQueryResult][:queryStatus]
      @result_location = json_body[:data][:getFlexReportBulkQueryResult][:resultLocation]
      unless status.eql? 'COMPLETED'
        attempts = 0
        # allow up to 1 minute for job to complete
        wait_time_increment = 5
        retry_attempts = 12
        wait_time = wait_time_increment
        while status != 'COMPLETED' && attempts < retry_attempts
        wrap_request do
          variables = {
            flexReportExecutionId: query_id
          }
          graphql url(service: service),
            format_request(
              :getFlexReportBulkQueryResult,
              service: service
            ),
            variables, headers
          expect_success_graphql
          check(:getFlexReportBulkQueryResult, service: service)
          expect(json_body[:data][:getFlexReportBulkQueryResult][:queryStatus]).to_not eq("FAILED")
        end
        status = json_body[:data][:getFlexReportBulkQueryResult][:queryStatus]
        @result_location = json_body[:data][:getFlexReportBulkQueryResult][:resultLocation]
        attempts += 1
        sleep(wait_time_increment)
        wait_time += wait_time_increment
        end
      end
      expect(status).to eq("COMPLETED")
      expect(@result_location.length).to be > 0
    end
  end
end
