require_relative 'spec_helper'

module MdrTest
  describe 'Mdr previewFlexReport' do
    include_context 'request metadata'

    before(:all) do
      @hourly_relative_hours = 720
      @weekly_relative_weeks = 51
      @daily_relative_days = 63
      @monthly_relative_months = 13
      @interactive_api = InteractiveApi.new
      @prepared_dataset_helper = PreparedDatasetHelper.new
      @queries_by_granularity = { 'hourly_sql' => "SELECT timeInterval_Hour AS Hour, SUM(cost) AS SUM_cost,  \"AWS-Account\" AS AWS_Account, \"AWS-Billing-Account\" AS AWS_Billing_Account, \"AWS-Service-Category\" AS AWS_Service_Category, Services.\"Services\" AS Services_Services FROM AWS_COST_HISTORY GROUP BY timeInterval_Hour, \"AWS-Account\", \"AWS-Billing-Account\", \"AWS-Service-Category\", Services.\"Services\" ORDER BY Hour ASC",
                                  'weekly_sql' => "SELECT timeInterval_Week AS Week, SUM(cost) AS SUM_cost, \"AWS-Account\" AS AWS_Account, \"AWS-Billing-Account\" AS AWS_Billing_Account, \"AWS-Service-Category\" AS AWS_Service_Category, Services.Services AS Services_Services FROM AWS_COST_HISTORY GROUP BY timeInterval_Week, \"AWS-Account\", \"AWS-Billing-Account\", \"AWS-Service-Category\", Services.Services ORDER BY Week ASC",
                                  'daily_sql' => "SELECT timeInterval_Day AS Day, SUM(cost) AS SUM_cost,  \"AWS-Account\" AS AWS_Account, \"AWS-Billing-Account\" AS AWS_Billing_Account, \"AWS-Service-Category\" AS AWS_Service_Category, Services.Services AS Services_Services FROM AWS_COST_HISTORY GROUP BY timeInterval_Day, \"AWS-Account\", \"AWS-Billing-Account\", \"AWS-Service-Category\", Services.Services ORDER BY Day ASC",
                                  'monthly_sql' => "SELECT timeInterval_Month AS Month, SUM(cost) AS SUM_cost, \"AWS-Account\" AS AWS_Account, \"AWS-Billing-Account\" AS AWS_Billing_Account, \"AWS-Service-Category\" AS AWS_Service_Category, Services.Services AS Services_Services FROM AWS_COST_HISTORY GROUP BY timeInterval_Month, \"AWS-Account\", \"AWS-Billing-Account\", \"AWS-Service-Category\", Services.Services ORDER BY Month ASC"}
      # TEMPORARILY COMMENTED DUE TO COLUMN NAME CHANGES
      #@queries_by_granularity = { 'hourly_sql' => "SELECT timeInterval_Hour AS Hour, SUM(cost) AS SUM_cost, SUM(billing_rule_savings) AS SUM_billing_rule_savings, SUM(cost_without_billing_rules) AS SUM_cost_without_billing_rules, \"AWS-Account\" AS AWS_Account, \"AWS-Billing-Account\" AS AWS_Billing_Account, \"AWS-Service-Category\" AS AWS_Service_Category, Services.\"Services\" AS Services_Services FROM AWS_COST_HISTORY GROUP BY timeInterval_Hour, \"AWS-Account\", \"AWS-Billing-Account\", \"AWS-Service-Category\", Services.\"Services\" ORDER BY Hour ASC",
      #                            'weekly_sql' => "SELECT timeInterval_Week AS Week, SUM(cost) AS SUM_cost, SUM(billing_rule_savings) AS SUM_billing_rule_savings, SUM(cost_without_billing_rules) AS SUM_cost_without_billing_rules, \"AWS-Account\" AS AWS_Account, \"AWS-Billing-Account\" AS AWS_Billing_Account, \"AWS-Service-Category\" AS AWS_Service_Category, Services.Services AS Services_Services FROM AWS_COST_HISTORY GROUP BY timeInterval_Week, \"AWS-Account\", \"AWS-Billing-Account\", \"AWS-Service-Category\", Services.Services ORDER BY Week ASC",
      #                            'daily_sql' => "SELECT timeInterval_Day AS Day, SUM(cost) AS SUM_cost, SUM(billing_rule_savings) AS SUM_billing_rule_savings, SUM(cost_without_billing_rules) AS SUM_cost_without_billing_rules, \"AWS-Account\" AS AWS_Account, \"AWS-Billing-Account\" AS AWS_Billing_Account, \"AWS-Service-Category\" AS AWS_Service_Category, Services.Services AS Services_Services FROM AWS_COST_HISTORY GROUP BY timeInterval_Day, \"AWS-Account\", \"AWS-Billing-Account\", \"AWS-Service-Category\", Services.Services ORDER BY Day ASC",
      #                            'monthly_sql' => "SELECT timeInterval_Month AS Month, SUM(cost) AS SUM_cost, SUM(billing_rule_savings) AS SUM_billing_rule_savings, SUM(cost_without_billing_rules) AS SUM_cost_without_billing_rules, \"AWS-Account\" AS AWS_Account, \"AWS-Billing-Account\" AS AWS_Billing_Account, \"AWS-Service-Category\" AS AWS_Service_Category, Services.Services AS Services_Services FROM AWS_COST_HISTORY GROUP BY timeInterval_Month, \"AWS-Account\", \"AWS-Billing-Account\", \"AWS-Service-Category\", Services.Services ORDER BY Month ASC"}
    end

    it 'can query cost history prepared dataset for different granualrities' do
      query_headers = headers
      query_headers = pre_prod_headers if ENV['TEST_ENV'].eql? 'preprod'
      failures = @prepared_dataset_helper.preview_flexreport_api(service, query_headers, @queries_by_granularity)
      expect(failures).to eql([])
    end

    xit "can executeFlexReportQuery and getFlexReportQueryResult for AWS Cost History Beta Dataset - HOURLY granularity", :prepared_dataset do
      query_headers = headers
      query_headers = pre_prod_headers if ENV['TEST_ENV'].eql? 'preprod'
      failures = @interactive_api.executeAndFetchFlexReportQuery(service, query_headers, 'HOURLY', nil, nil, @hourly_sql, 300, 100, true, false, 'false', 720 )
      expect(failures).to be_empty
    end

    xit "can executeFlexReportQuery and getFlexReportQueryResult for AWS Cost History Beta Dataset - DAILY granularity", :prepared_dataset do
      query_headers = headers
      query_headers = pre_prod_headers if ENV['TEST_ENV'].eql? 'preprod'
      failures = @interactive_api.executeAndFetchFlexReportQuery(service, query_headers, 'DAILY', nil, nil, @daily_sql, 300, 100, true, false, 'false', 90 )
      expect(failures).to be_empty
    end

    xit "can executeFlexReportQuery and getFlexReportQueryResult for AWS Cost History Beta Dataset - WEEKLY granularity", :prepared_dataset do
      query_headers = headers
      query_headers = pre_prod_headers if ENV['TEST_ENV'].eql? 'preprod'
      failures = @interactive_api.executeAndFetchFlexReportQuery(service, query_headers, 'WEEKLY', nil, nil, @weekly_sql, 300, 100, true, false, 'false', 51 )
      expect(failures).to be_empty
    end

    xit "can executeFlexReportQuery and getFlexReportQueryResult for AWS Cost History Beta Dataset - MONTHLY granularity", :prepared_dataset do
      query_headers = headers
      query_headers = pre_prod_headers if ENV['TEST_ENV'].eql? 'preprod'
      failures = @interactive_api.executeAndFetchFlexReportQuery(service, query_headers, 'MONTHLY', nil, nil, @monthly_sql, 300, 100, true, false, 'false', 12 )
      expect(failures).to be_empty
    end
  end
end
