require 'cht-chaphql'
require 'cht_apitest_local_auth'
require 'cht_apitest_local_auth/service'

Dir[File.join(__dir__, '../lib/*', '*.rb')].each { |file| require file }
Dir[File.join(__dir__, '../lib/helpers/data/*', '*.json')].each { |file| require file }
Dir[File.join(__dir__, 'helpers/*.rb')].each { |f| require f }

RSpec.configure do |c|
  c.include Chaphql
  c.include Chaphql::GraphQLHelpers
  c.include Chaphql::AnalyzerHelpers
  c.include Chaphql::StringManipulation
  c.include Chaphql::ServiceHelper
  c.include Chaphql::IntrospectConflict
  c.include ChtApitestLocalAuth
  c.include ChtApitestAuthCommon
  c.include Spaceborne
  include ChtApitestMdr
  include MdrHelpers
  include MdrTest
  c.filter_run_excluding(save_schema: true)
  c.filter_run_excluding(staging_env: true) unless ENV['TEST_ENV'] == 'staging'
  c.filter_run_excluding(not_via_proxy: true) if ENV['EKS_ENV'] == 'external'
  c.shared_context_metadata_behavior = :apply_to_host_groups
end

RSpec.shared_context 'request metadata', shared_context: :metadata do
  let(:start_time) { Time.now.to_i }
  let(:via_public) { %w[staging canary prod preprod].include?(getenv) }
  let(:service) { ChtApitestMdr::MdrService.new }
  let(:auth_service) { LocalAuthService.new(via_public: via_public) }
  let(:tokens) { login_api(ENV['API_KEY'], service: auth_service) }
  let(:headers) do
    apitest_headers(with_auth: true, access_token: tokens[:access_token])
  end
  if ENV['TEST_ENV'].eql? 'preprod'
    let!(:tokens_preprod) { login_api(ENV['WS3_PREPROD_API_KEY'], service: auth_service) }
    let!(:pre_prod_headers) do
      apitest_headers(with_auth: true, access_token: tokens_preprod[:access_token])
    end
  end
  let(:dbg_headers) { headers.merge(use_curl: 'debug') }
  let(:get_new_tokens) do
    headers = apitest_headers(auth_service: :auth_service,
                              with_auth: true,
                              refresh: tokens[:refresh_token])
    dbg_headers = headers.merge(use_curl: 'debug')
  end
  COLUMN_METADATA = File.read(File.join(__dir__,'../lib/helpers/data/metadata_expected.json'))
end

RSpec.configure do |c|
  c.include_context 'request metadata', include_shared: true
end
